#pragma once


#include <allegro_flare/sound_object.h>
#include <AllegroFlare/SampleBin.hpp>
#include <DragonWrath/MusicAndSoundEffectTrackNames.hpp>
#include <map>


namespace DragonWrath
{
   class AudioController
   {
   private:
      AllegroFlare::SampleBin &sample_bin;
      std::string sound_effects_identifier_prefix;
      std::string music_tracks_identifier_prefix;
      std::vector<AudioRepositoryElement> sound_effect_elements;
      std::vector<AudioRepositoryElement> music_track_elements;
      std::map<int, Sound*> sound_effects;
      std::map<int, Sound*> music_tracks;
      int current_music_track_id;
      float global_volume;

      bool output_loading_debug_to_cout;

      Sound *find_sound_effect_by_id(int id);
      Sound *find_music_track_by_id(int id);

   public:
      AudioController(
            AllegroFlare::SampleBin &sample_bin,
            std::vector<AudioRepositoryElement> music_track_elements,
            std::vector<AudioRepositoryElement> sound_effect_elements
         );
      ~AudioController();

      void initialize();
      void stop_all();

      void set_global_volume(float volume);

      void play_music_track_by_id(int id);
      void play_sound_effect_by_id(int id);
   };
}


